/*
 * Copyright (c) 2006-2013 DMDirc Developers
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.dmdirc.addons.urlcatcher;

import com.dmdirc.FrameContainer;
import com.dmdirc.Raw;
import com.dmdirc.actions.CoreActionType;
import com.dmdirc.interfaces.actions.ActionType;
import com.dmdirc.interfaces.ActionController;
import com.dmdirc.interfaces.ActionListener;
import com.dmdirc.interfaces.CommandController;
import com.dmdirc.interfaces.ConfigChangeListener;
import com.dmdirc.interfaces.IdentityController;
import com.dmdirc.plugins.implementations.BaseCommandPlugin;
import com.dmdirc.ui.messages.Styliser;

import java.util.HashMap;
import java.util.Map;

/**
 * A plugin which logs all observed URLs and allows the user to list them
 * later.
 */
public class UrlCatcherPlugin extends BaseCommandPlugin implements
        ActionListener, ConfigChangeListener {

    /* URLs and the number of times they were mentioned. */
    private final Map<String, Integer> urls = new HashMap<>();
    /** Controller to read settings from. */
    private final IdentityController identityController;
    /** Controller to listen to action events from. */
    private final ActionController actionController;
    /** Whether to capture URLs from the raw window. */
    private boolean captureRaw = false;

    /**
     * Creates a new instance of this plugin.
     *
     * @param commandController Command controller to register commands
     * @param identityController Controller to load settings from.
     * @param actionController Controller to listen to action events from.
     */
    public UrlCatcherPlugin(
            final CommandController commandController,
            final IdentityController identityController,
            final ActionController actionController) {
        super(commandController);
        this.identityController = identityController;
        this.actionController = actionController;
        registerCommand(new UrlListCommand(this), UrlListCommand.INFO);
    }

    /** {@inheritDoc} */
    @Override
    public void onLoad() {
        actionController.registerListener(this, CoreActionType.CLIENT_LINE_ADDED);
        identityController.getGlobalConfiguration().addChangeListener(getDomain(), this);
        updateConfig();
        super.onLoad();
    }

    /** {@inheritDoc} */
    @Override
    public void onUnload() {
        actionController.unregisterListener(this);
        identityController.getGlobalConfiguration().removeListener(this);
        super.onUnload();
    }

    /**
     * Re-reads the configuration of the URL catcher plugin.
     */
    private void updateConfig() {
        captureRaw = identityController.getGlobalConfiguration().getOptionBool(getDomain(), "captureraw");
    }

    /** {@inheritDoc} */
    @Override
    public void processEvent(final ActionType type, final StringBuffer format,
            final Object... arguments) {
        if (arguments[0] instanceof Raw && !captureRaw) {
            return;
        }

        final String message = ((FrameContainer) arguments[0]).getStyliser()
                .doLinks((String) arguments[1]);

        if (message.indexOf(Styliser.CODE_HYPERLINK) > -1) {
            final String[] parts = message.split(Character.toString(Styliser.CODE_HYPERLINK));

            for (int i = 1; i < parts.length; i += 2) {
                addURL(parts[i]);
            }
        }
    }

    /**
     * Adds an URL to the list of tracked URLs, or increases its counter by one.
     *
     * @param url The URL to be added
     */
    private void addURL(final String url) {
        if (urls.containsKey(url)) {
            urls.put(url, urls.get(url) + 1);
        } else {
            urls.put(url, 1);
        }
    }

    /**
     * Retrieves the URLs that have been recorded.
     *
     * @return A map of URLs to the number of times they've been seen.
     */
    public Map<String, Integer> getURLS() {
        return urls;
    }

    /** {@inheritDoc} */
    @Override
    public void configChanged(final String domain, final String key) {
        updateConfig();
    }

}
